/**
 * Enrollment GUI Class.
 * Author: Dr. Manal Helal
 * Course: CS243 - Object Oriented Programming
 * Date: 29 December 2013
 * Revision 1
 * 
 * There is a bug in the student list when the course changes, first student is always selected whether he/she enrolled in the course or not.
 * Simple File Manager for a small application to manage a school.
 * 
 * */

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.awt.GridLayout;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.ListSelectionModel;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;


public class EnrollmentGUI extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JPanel contentPane;
	private ArrayList<Student> studArray = null;
	private ArrayList<Course> courseArray = null;
	private ArrayList<Lecturer> lectArray = null;
	private ArrayList<TA> taArray = null;
	JList<String>  jlStudents;
	
	/**
	 * Create the frame.
	 */
	public EnrollmentGUI() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				String[] data= {"Code", "Name"}; 
				FM.writeToTabDelimitedFile(data, FM.enrollFileName);
				for (int i=0; i<courseArray.size();i++) {
					Course c = courseArray.get(i);
					data[0] = c.getCode();
					for (int j=0; j<c.getStudents().size();j++) {
						Student s = c.getStudents().get(j);
						data[1] = s.getLName() + ", "+ s.getMI() + ", " + s.getFName();
					FM.appendToTabDelimitedFile(data, FM.enrollFileName);
					}
				}
				
			}
		});
		lectArray = FM.readLecturer();
		taArray = FM.readTAs();
		studArray = FM.readStudents();
		courseArray = FM.readCourses(lectArray, taArray);
		courseArray = FM.readEnrollment(courseArray, studArray);
		
		
		setTitle("Enrollment Management");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 465, 242);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(new GridLayout(3, 2, 3, 0));
		
		JLabel lblCourse = new JLabel("Course");
		contentPane.add(lblCourse);
		
		Vector<String>  lItems=new Vector<String> ();
		for (int i = 0; i < courseArray.size();i++) 
			lItems.add(courseArray.get(i).getCode() + ": " + courseArray.get(i).getName());
		
		final DefaultComboBoxModel<String> courseModel = new DefaultComboBoxModel<String>(lItems);
		
		final JComboBox<String> cbCourses = new JComboBox<String>(courseModel);
		
		
		contentPane.add(cbCourses);
		
		JLabel lblStudent = new JLabel("Student");
		contentPane.add(lblStudent);
		
		
	    
		lItems = new Vector<String> ();
		
		for (int i = 0; i < studArray.size();i++) 
			lItems.add(studArray.get(i).getLName() + ", "+ studArray.get(i).getMI() + ", " + studArray.get(i).getFName());
	   
		final DefaultComboBoxModel<String> stModel = new DefaultComboBoxModel<String>(lItems);
	    
		jlStudents = new JList<String> (stModel);
	    jlStudents.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		contentPane.add(jlStudents);

		cbCourses.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Course c = courseArray.get(cbCourses.getSelectedIndex());
				int indices[] = new int[1];
				for (int i = 0; i < c.getStudents().size();i++) {
					Student s = c.getStudents().get(i);
					for (int j = 0; j < stModel.getSize();j++) {
						String[] SNames = ((String) stModel.getElementAt(j)).split(", ");
						if (SNames[0].equals(s.getLName()) && SNames[1].equals(s.getMI()) && SNames[2].equals(s.getFName())) {
							int newindices[] = new int[indices.length+1];
							System.arraycopy(indices, 0, newindices, 0, indices.length);
							newindices[indices.length] = j;
							indices = newindices;
						}
					}					
				}
				jlStudents.setSelectedIndices(indices);

			}
		});
		
		JButton btnEnroll = new JButton("Enroll");
		btnEnroll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Course course = courseArray.get(cbCourses.getSelectedIndex());
				Object[] selStudents = jlStudents.getSelectedValues(); 
				for (int i = 0 ; i < selStudents.length; i++) {
					String[] SNames = ((String) selStudents[i]).split(", ");
					
					Student s = FM.getStudent (studArray, SNames[2], SNames[1], SNames[0]);
					course.enrollStudent(s);
				}
			}
		});
		contentPane.add(btnEnroll);
		
		JButton btnWithdraw = new JButton("Withdraw");
		btnWithdraw.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Course course = courseArray.get(cbCourses.getSelectedIndex());
				Object[] selStudents = jlStudents.getSelectedValues(); 
				for (int i = 0 ; i < selStudents.length; i++) {
					String[] SNames = ((String) selStudents[i]).split(", ");
					
					Student s = FM.getStudent (studArray, SNames[2], SNames[1], SNames[0]);
					course.withdrawStudent(s);
				}
				jlStudents.setSelectedIndex(-1);
			}
		});
		contentPane.add(btnWithdraw);
	}

}
