/**
 * File Management Class
 * Author: Dr. Manal Helal
 * Course: CS243 - Object Oriented Programming
 * Date: 29 December 2013
 * Revision 1
 * 
 * Simple File Manager for a small application to manage a school.
 * 
 * */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;



public class FM {
	
	public static final String schoolFileName = "db/school.txt";
	public static final String lecturerFileName = "db/lecturer.txt";
	public static final String taFileName = "db/ta.txt";
	public static final String courseFileName = "db/course.txt";
	public static final String studentFileName = "db/student.txt";
	public static final String roomFileName = "db/room.txt";
	public static final String enrollFileName = "db/enrollment.txt";
	
	/**
	 * Method: writeToTabDelimitedFile
	 * Input: String Array of Data to be printed in the file as one row tab delimited
	 * Output: Boolean variable signaling success or failure of the writing to the file.
	 * 
	 * */

	public static boolean writeToTabDelimitedFile(String[] data, String FileName) {
		PrintWriter out = null;
		try {
			File file =new File(FileName);
			File path =new File(file.getParent());
			if (!path.exists())
				path.mkdirs();
			out =new PrintWriter(new BufferedWriter(new FileWriter(FileName))); // loop through all your data and print it to the file
			for (int i=0;i< data.length;i++) 
				out.write(data[i]+"\t"); 
			out.write("\n");
		} 
		catch (IOException e) {
			System.out.println("Error Printing Tab Delimited File");
			return false; 
		}
		finally {
			out.close(); 
		}
		return true; 
	}
	
	/**
	 * Method: appendToTabDelimitedFile
	 * Input: String Array of Data to be appended to an existing file as one row tab delimited
	 * Output: Boolean variable signaling success or failure of the writing to the file.
	 * 
	 * */
	public static boolean appendToTabDelimitedFile (String[] data, String FileName) { 
		PrintWriter out = null;
		try{
			File file =new File(FileName);
			if(!file.exists()) //if file doesn’t exists, return false
				return false;
			out =new PrintWriter(new BufferedWriter(new FileWriter(FileName, true)));
			 
			for (int i=0;i< data.length;i++)
				out.write(data[i]+"\t"); 
			out.write("\n");
			
		} 
		catch (IOException e){ 
			e.printStackTrace();
		}
		finally {
			out.close();
		}
		return true; 
	}
	
	/**
	 * Method: deleteFile
	 * Input: filename to be deleted
	 * Output: Boolean variable signaling success or failure of the writing to the file.
	 * 
	 * */
	public static boolean deleteFile (String FileName) {
		File file =new File(FileName);
		if (file.exists()) {
			file.delete();
			return true;
		}
		else		
			return false;
		
	}
	
	/**
	 * Method: readFromTabDelimitedFile
	 * Input: filename to be read
	 * Output: String array of lines read. In further functions, each line is a tab delimited values of the file format followed in the file being read.
	 * 
	 * */
	public static String [] readFromTabDelimitedFile (String FileName) { 
		List<String> lines = new ArrayList<String>();
		FileReader fileReader  = null;
		BufferedReader bufferedReader = null;
		try {
			fileReader = new FileReader(FileName); 
			bufferedReader = new BufferedReader(fileReader); 
			String line = null;
			while ((line = bufferedReader.readLine()) != null)
				lines.add(line); bufferedReader.close();
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
			return null;
		}
		catch (IOException e) { 
			e.printStackTrace();
			return null;
		}
		finally {
			try {
				if (bufferedReader != null)	
					bufferedReader.close();
				if (fileReader != null)	
					fileReader.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return null;
			}
			
		}
		return lines.toArray(new String[lines.size()]); 
	}
	
	/**
	 * Method: readStudents 
	 * Input: filename to be read
	 * Output: using readFromTabDelimitedFile to read a string array of lines read from file, 
	 *   each line is a tab delimited values of the Student information. Student Information is extracted using String.split and an ArrayList<Student> is returned
	 * 
	 * */
	static ArrayList<Student> readStudents () {
		ArrayList<Student> studArray = new ArrayList<Student>();
		Student student = null;
		String[] readData = FM.readFromTabDelimitedFile(FM.studentFileName); 
		if (readData == null) {
			return null;
		}
		if (readData.length < 2) {
			System.out.println("TA text file format error. Ignoring.");
			return null;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			student = new Student (null, null, null, null, null, null, null, null);
			if (tokens.length >= 1) {
				student.setFName(tokens[0]);
			}
			if (tokens.length >= 2) {
				student.setMI(tokens[1]);
			}
			if (tokens.length >= 3) {
				student.setLName(tokens[2]);
			}
			if (tokens.length >= 4) {
				student.setPhone(tokens[3]);
			}
			if (tokens.length >= 5)  {
				student.setMobile(tokens[4]);
			}
			if (tokens.length >= 6) {
				student.setFax(tokens[5]);
			}
			if (tokens.length >= 7) {
				student.setEmail(tokens[6]);
			}
			if (tokens.length >= 8) {
				student.setAddress(tokens[7]);
			}
			studArray.add(student);
		} 			
		return studArray;
	}
	
	/**
	 * Method: readRoom 
	 * Input: filename to be read
	 * Output: using readFromTabDelimitedFile to read a string array of lines read from file, 
	 *   each line is a tab delimited values of the room information. Room Information is extracted using String.split and an ArrayList<Room> is returned
	 * 
	 * */
	static ArrayList<Room> readRoom () {
		ArrayList<Room> roomArray = new ArrayList<Room>();
		Room room = null;
		String[] readData = FM.readFromTabDelimitedFile(FM.roomFileName); 
		if (readData == null) {
			return null;
		}
		if (readData.length < 2) {
			System.out.println("Room text file format error. Ignoring.");
			return null;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			room = new Room (null);
			if (tokens.length >= 1) {
				room.setName(tokens[0]);
			}
			roomArray.add(room);
		} 			
		return roomArray;
	}
	
	/**
	 * Method: readLecturer 
	 * Input: filename to be read
	 * Output: using readFromTabDelimitedFile to read a string array of lines read from file, 
	 *   each line is a tab delimited values of the Lecturer information. Lecturer Information is extracted using String.split and an ArrayList<Lecturer> is returned
	 * 
	 * */
	static ArrayList<Lecturer> readLecturer () {
		ArrayList<Lecturer> lectArray = new ArrayList<Lecturer>();
		Lecturer lecturer = new Lecturer (null, null, null, null, null, null, null, null);
		
		String[] readData = FM.readFromTabDelimitedFile(FM.lecturerFileName); 
		if (readData == null) {
			return null;
			
		}
		if (readData.length < 2) {
			System.out.println("Lecturer text file format error. Ignoring.");
			return null;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			lecturer = new Lecturer (null, null, null, null, null, null, null, null);
			if (tokens.length >= 1) {
				lecturer.setFName(tokens[0]);
			}
			if (tokens.length >= 2) {
				lecturer.setMI(tokens[1]);
			}
			if (tokens.length >= 3) {
				lecturer.setLName(tokens[2]);
			}
			if (tokens.length >= 4) {
				lecturer.setPhone(tokens[3]);
			}
			if (tokens.length >= 5)  {
				lecturer.setMobile(tokens[4]);
			}
			if (tokens.length >= 6) {
				lecturer.setFax(tokens[5]);
			}
			if (tokens.length >= 7) {
				lecturer.setEmail(tokens[6]);
			}
			if (tokens.length >= 8) {
				lecturer.setAddress(tokens[7]);
			}
			lectArray.add(lecturer);
		}
		return lectArray; 				
	}
	
	/**
	 * Method: readTAs 
	 * Input: filename to be read
	 * Output: using readFromTabDelimitedFile to read a string array of lines read from file, 
	 *   each line is a tab delimited values of the TA information. TA Information is extracted using String.split and an ArrayList<TA> is returned
	 * 
	 * */
	static ArrayList<TA> readTAs () {
		TA ta = new TA (null, null, null, null, null, null, null, null);
		ArrayList<TA> taArray = new ArrayList<TA>();
		String[] readData = FM.readFromTabDelimitedFile(FM.taFileName); 
		if (readData == null) {
			return null;
		}
		if (readData.length < 2) {
			System.out.println("TA text file format error. Ignoring.");
			return null;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			ta = new TA (null, null, null, null, null, null, null, null);
			if (tokens.length >= 1) {
				ta.setFName(tokens[0]);
			}
			if (tokens.length >= 2) {
				ta.setMI(tokens[1]);
			}
			if (tokens.length >= 3) {
				ta.setLName(tokens[2]);
			}
			if (tokens.length >= 4) {
				ta.setPhone(tokens[3]);
			}
			if (tokens.length >= 5)  {
				ta.setMobile(tokens[4]);
			}
			if (tokens.length >= 6) {
				ta.setFax(tokens[5]);
			}
			if (tokens.length >= 7) {
				ta.setEmail(tokens[6]);
			}
			if (tokens.length >= 8) {
				ta.setAddress(tokens[7]);
			}
			taArray.add(ta);
		} 	
		return taArray;
	}
	
	/**
	 * Method: readCourses 
	 * Input: filename to be read
	 * Output: using readFromTabDelimitedFile to read a string array of lines read from file, 
	 *   each line is a tab delimited values of the Course information. Course Information is extracted using String.split and an ArrayList<Course> is returned
	 * 
	 * */
	static ArrayList<Course> readCourses (ArrayList<Lecturer> lectArray, ArrayList<TA> taArray) {
		ArrayList<Course> courseArray = new ArrayList<Course>();
		Course course = new Course (null, null, null, null, null, Day.FRIDAY, 0, null, Day.FRIDAY, 0, null);
		
		String[] readData = FM.readFromTabDelimitedFile(FM.courseFileName); 
		if (readData == null) {
			return null;
		}
		if (readData.length < 2) {
			System.out.println("Course text file format error. Ignoring.");
			return null;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			course = new Course (null, null, null, null, null, Day.FRIDAY, 0, null, Day.FRIDAY, 0, null);
			if (tokens.length >= 1) {
				course.setCode(tokens[0]);
			}
			if (tokens.length >= 2) {
				course.setName(tokens[1]);
			}
			if (tokens.length >= 3) {
				String lNames[] = tokens[2].split(", ");
				Lecturer l = FM.getLecturer(lectArray, lNames[2], lNames[1], lNames[0]);
				course.setLecturer(l);
			}
			if (tokens.length >= 4) {
				String lNames[] = tokens[3].split(", ");
				TA t = FM.getTA(taArray, lNames[2], lNames[1], lNames[0]);
				course.setTA(t);
			}
			if (tokens.length >= 5)  {
				course.setLecureDay(course.getDay(tokens[4]));
			}
			if (tokens.length >= 6) {
				course.setLecturePeriod(Integer.parseInt(tokens[5].trim()));
			}
			if (tokens.length >= 7) {
				course.setLectureRoom(tokens[6]);
			}
			if (tokens.length >= 8)  {
				course.setLabDay(course.getDay(tokens[7]));
			}
			if (tokens.length >= 9) {
				course.setLabPeriod(Integer.parseInt(tokens[8].trim()));
			}
			if (tokens.length >= 10) {
				course.setLabRoom(tokens[9]);
			}
			courseArray.add(course);
		} 
		return courseArray;
	}
	
	/**
	 * Method: readEnrollment 
	 * Input: course and student Arrays
	 * Output: updated course Array by enrolling students saved to file to be enrolled in the course code.
	 * 
	 * */
	static ArrayList<Course> readEnrollment (ArrayList<Course> courseArray, ArrayList<Student> studArray) {
		
		
		String[] readData = FM.readFromTabDelimitedFile(FM.enrollFileName); 
		if (readData == null) {
			return courseArray;
		}
		if (readData.length < 2) {
			System.out.println("Enrollment text file format error. Ignoring.");
			return courseArray;
		}
		for (int i = 1;i<readData.length;i++) {
			String [] tokens = readData[i].split("\t"); 
			Course course = getCourse(courseArray, tokens[0]);
			
			
			if (tokens.length >= 2) {
				String lNames[] = tokens[1].split(", ");
				Student s = FM.getStudent (studArray, lNames[2], lNames[1], lNames[0]);
				course.enrollStudent(s);
			}
		} 
		return courseArray;
	}
	
	/**
	 * Method: getLecturer 
	 * Input: lecturer ArrayList and Lecturer First Name, MI, and Last Name
	 * Output: Lecturer Object matching the name searched from the ArrayList<Lecturer>, null if not found
	 * 
	 * */
	static Lecturer getLecturer (ArrayList<Lecturer> lectArray, String FName, String MI, String LName) {
		for (int i = 0; i < lectArray.size(); i++)
			if (lectArray.get(i).getFName().equals(FName) &&  lectArray.get(i).getMI().equals(MI) && lectArray.get(i).getLName().equals(LName))
				return lectArray.get(i);
		return null;
	}
	
	/**
	 * Method: getTA 
	 * Input: TA ArrayList and TA First Name, MI, and Last Name
	 * Output: TA Object matching the name searched from the ArrayList<TA>, null if not found
	 * 
	 * */
	static TA getTA (ArrayList<TA> taArray, String FName, String MI, String LName) {
		for (int i = 0; i < taArray.size(); i++)
			if (taArray.get(i).getFName().equals(FName) &&  taArray.get(i).getMI().equals(MI) && taArray.get(i).getLName().equals(LName))
				return taArray.get(i);
		return null;
	}
	
	/**
	 * Method: getStudent 
	 * Input: Student ArrayList and Student First Name, MI, and Last Name
	 * Output: Student Object matching the name searched from the ArrayList<Student>, null if not found
	 * 
	 * */
	static Student getStudent (ArrayList<Student> studArray, String FName, String MI, String LName) {
		for (int i = 0; i < studArray.size(); i++)
			if (studArray.get(i).getFName().equals(FName) &&  studArray.get(i).getMI().equals(MI) && studArray.get(i).getLName().equals(LName))
				return studArray.get(i);
		return null;
	}
	
	/**
	 * Method: getCourse 
	 * Input: Course ArrayList and Course Code
	 * Output: Course Object matching the code searched from the ArrayList<Course>, null if not found
	 * 
	 * */
	static Course getCourse (ArrayList<Course> courseArray, String code) {
		for (int i = 0; i < courseArray.size(); i++)
			if (courseArray.get(i).getCode().equals(code))
				return courseArray.get(i);
		return null;
	}

}
