/**
 * School Management System GUI Class
 * Author: Dr. Manal Helal
 * Course: CS243 - Object Oriented Programming
 * Date: 29 December 2013
 * Revision 1
 * 
 * Simple File Manager for a small application to manage a school.
 * 
 * */

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import javax.swing.JTextField;
import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


public class SchoolGUI {

	private JFrame frmSchoolManagementSystem;
	private JTextField jtfInstituteName;
	private JLabel lblNewLabel_1;
	private JTextField jtfSchool;
	private JLabel lblNewLabel_2;
	private JTextField jtfCurrentSemester;
	private JLabel lblNewLabel_3;
	private JTextField jtfSemesterStartDate;
	private JLabel lblNewLabel_4;
	private JTextField jtfSemesterEndDate;
	private JButton btnUpdate;
	private JButton btnDelete;
	private JButton btnLecturers;
	private JButton btnTAs;
	private JButton btnCourses;
	private JButton btnStudents;
	private JButton btnRooms;
	private JButton btnEnrollment;

	
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					SchoolGUI window = new SchoolGUI();
					window.frmSchoolManagementSystem.setVisible(true);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public SchoolGUI() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		School school = new School();
		
		frmSchoolManagementSystem = new JFrame();
		frmSchoolManagementSystem.addWindowListener(new WindowAdapter() {
			@Override
			public void windowOpened(WindowEvent e) {
				String[] readData = FM.readFromTabDelimitedFile("db/school.txt"); 
				if (readData == null) {
					JOptionPane.showMessageDialog(frmSchoolManagementSystem, "Please Enter School Details", "Warnning", JOptionPane.WARNING_MESSAGE);
				}
				if (readData.length < 2) {
					System.out.println("School text file format error. Ignoring.");
					return;
				}
				else {
					String [] tokens = readData[1].split("\t"); 
					if (tokens.length >= 1)
						jtfInstituteName.setText(tokens[0]);
					if (tokens.length >= 2)
						jtfSchool.setText(tokens[1]);
					if (tokens.length >= 3)
						jtfCurrentSemester.setText(tokens[2]); 
					if (tokens.length >= 4)
						jtfSemesterStartDate.setText(tokens[3]); 
					if (tokens.length >= 5)
						jtfSemesterEndDate.setText(tokens[4]); 
				} 
			}
		});
		frmSchoolManagementSystem.setTitle("School Management System");
		frmSchoolManagementSystem.setBounds(100, 100, 547, 164);
		frmSchoolManagementSystem.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmSchoolManagementSystem.getContentPane().setLayout(new GridLayout(5, 4, 0, 0));
		
		
		JLabel lblNewLabel = new JLabel("Institution Name");
		frmSchoolManagementSystem.getContentPane().add(lblNewLabel);
		
		jtfInstituteName = new JTextField();
		frmSchoolManagementSystem.getContentPane().add(jtfInstituteName);
		jtfInstituteName.setColumns(10);
		
		lblNewLabel_1 = new JLabel("School Name");
		frmSchoolManagementSystem.getContentPane().add(lblNewLabel_1);
		
		jtfSchool = new JTextField();
		frmSchoolManagementSystem.getContentPane().add(jtfSchool);
		jtfSchool.setColumns(10);
		
		lblNewLabel_2 = new JLabel("Current Semester");
		frmSchoolManagementSystem.getContentPane().add(lblNewLabel_2);
		
		jtfCurrentSemester = new JTextField();
		frmSchoolManagementSystem.getContentPane().add(jtfCurrentSemester);
		jtfCurrentSemester.setColumns(10);
		
		lblNewLabel_3 = new JLabel("Semester Start Date");
		frmSchoolManagementSystem.getContentPane().add(lblNewLabel_3);
		
		jtfSemesterStartDate = new JTextField();
		frmSchoolManagementSystem.getContentPane().add(jtfSemesterStartDate);
		jtfSemesterStartDate.setColumns(10);
		
		lblNewLabel_4 = new JLabel("Semester End Date");
		frmSchoolManagementSystem.getContentPane().add(lblNewLabel_4);
		
		jtfSemesterEndDate = new JTextField();
		frmSchoolManagementSystem.getContentPane().add(jtfSemesterEndDate);
		jtfSemesterEndDate.setColumns(10);
		
		btnUpdate = new JButton("Update");
		btnUpdate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String[] data= {"Institute", "School","CurrentSemester", "SemesterStart", "SemesterEnd"}; 
				FM.writeToTabDelimitedFile(data, "db/school.txt");
				data[0] = jtfInstituteName.getText();
				data[1] = jtfSchool.getText();
				data[2] = jtfCurrentSemester.getText(); 
				data[3] = jtfSemesterStartDate.getText(); 
				data[4] = jtfSemesterEndDate.getText(); 
				FM.appendToTabDelimitedFile(data, "db/school.txt");
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnUpdate);

		btnDelete = new JButton("Delete");
		btnDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int reply = JOptionPane.showConfirmDialog(frmSchoolManagementSystem, "Are you sure you want to delete all database files?");
				if (reply == 0) {
					FM.deleteFile("db/school.txt");
					FM.deleteFile("db/school.txt");
				}
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnDelete);

		btnLecturers = new JButton("Lecturers");
		btnLecturers.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							LecturerGUI frame = new LecturerGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnLecturers);
		
		btnTAs = new JButton("TAs");
		btnTAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							TAGUI frame = new TAGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnTAs);
		
		btnCourses = new JButton("Courses");
		btnCourses.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							CourseGUI frame = new CourseGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnCourses);
		
		btnStudents = new JButton("Students");
		btnStudents.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							StudentsGUI frame = new StudentsGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnStudents);
		
		btnRooms = new JButton("Rooms");
		btnRooms.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							RoomGUI frame = new RoomGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnRooms);
		
		btnEnrollment = new JButton("Enrollment");
		btnEnrollment.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventQueue.invokeLater(new Runnable() {
					public void run() {
						try {
							EnrollmentGUI frame = new EnrollmentGUI();
							frame.setVisible(true);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				});
			}
		});
		frmSchoolManagementSystem.getContentPane().add(btnEnrollment);
		
		
	}

}
